import 'package:flutter/material.dart';
import 'package:hyperkyc_flutter/hyperkyc_doc_data.dart';
import 'package:hyperkyc_flutter/hyperkyc_face_data.dart';
import 'package:hyperkyc_flutter/hyperkyc_face_match_data.dart';

/// HyperKycData will contain partial or the final result data based on the result state.
class HyperKycData {
  /// Selected country name
  final String? selectedCountry;

  /// Selected document name
  final String? selectedDocument;

  /// List of document capture and OCR results based on no. of sides of the selected document
  final List<DocData?>? docDataList;

  /// Face capture and liveness result
  final FaceData? faceData;

  /// Face match result
  final FaceMatchData? faceMatchData;

  /// Constructor
  HyperKycData({
    this.selectedCountry,
    this.selectedDocument,
    this.docDataList,
    this.faceData,
    this.faceMatchData,
  });

  /// Convert HyperKycData object to Map
  Map<String, dynamic> toMap() {
    return {
      'selectedCountry': this.selectedCountry,
      'selectedDocument': this.selectedDocument,
      'docDataList': this.docDataList,
      'faceData': this.faceData,
      'faceMatchData': this.faceMatchData,
    };
  }

  /// Convert Map to HyperKycData object
  factory HyperKycData.fromMap(Map<dynamic, dynamic> map) {
    if (map.isEmpty) {
      return HyperKycData();
    }

    List<DocData> docDataList = List.empty(growable: true);
    List docDataListFromMap = map['docDataList'] ?? [];
    for (var docDataMap in docDataListFromMap) {
      DocData docData = DocData.fromMap(docDataMap ?? {});
      docDataList.add(docData);
    }
    debugPrint('selectedCountry : ${map['selectedCountry']}');
    debugPrint('selectedDocument : ${map['selectedDocument']}');
    debugPrint('docDataList : ${map['docDataList'].toString()}');
    debugPrint('faceData : ${map['faceData'].toString()}');
    debugPrint('faceMatchData : ${map['faceMatchData'].toString()}');

    if (map['faceData'].isEmpty) {}
    return HyperKycData(
      selectedCountry: map['selectedCountry'] as String?,
      selectedDocument: map['selectedDocument'] as String?,
      docDataList: docDataList,
      faceData: FaceData.fromMap(map['faceData'] ?? {}),
      faceMatchData: FaceMatchData.fromMap(map['faceMatchData'] ?? {}),
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'HyperKycData{selectedCountry: $selectedCountry, selectedDocument: $selectedDocument, docDataList: $docDataList, faceData: $faceData, faceMatchData: $faceMatchData}';
  }
}
