import 'dart:convert';

import 'package:flutter/cupertino.dart';
import 'package:hyperkyc_flutter/hyperkyc_api_base_response.dart';

/// FaceMatchData contains face match result
class FaceMatchData {
  /// Action - pass / manualReview / fail
  final String? action;

  /// Response headers (json string) from face match API
  final String? responseHeaders;

  /// Response body from the face match API
  final FaceMatchApiDetail? responseResult;

  /// Constructor
  FaceMatchData({
    this.action,
    this.responseHeaders,
    this.responseResult,
  });

  /// Convert FaceMatchData object to Map
  Map<String, dynamic> toMap() {
    return {
      'action': this.action,
      'responseHeaders': this.responseHeaders,
      'responseResult': this.responseResult,
    };
  }

  /// Convert Map to FaceMatchData object
  factory FaceMatchData.fromMap(Map<dynamic, dynamic> map) {
    debugPrint('inside FaceMatchData.fromMap');
    if (map.isEmpty) {
      return FaceMatchData();
    }
    return FaceMatchData(
      action: map['action'] as String?,
      responseHeaders: map['responseHeaders'] as String?,
      responseResult:
          FaceMatchApiDetail.fromMap(jsonDecode(map['responseResult']) ?? {}),
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'FaceMatchData{action: $action, responseHeaders: $responseHeaders, responseResult: $responseResult}';
  }
}

/// FaceMatchApiDetail is response body from the Hyperverge face match API call
class FaceMatchApiDetail {
  /// Metadata contains data like TransactionId, RequestId, Attempts count, etc
  final Metadata? metadata;

  /// Status received after Hyperverge API call
  final String? status;

  /// StatusCode received after Hyperverge API call
  final String? statusCode;

  /// FaceMatchResult
  final FaceMatchResult? result;

  /// Constructor
  FaceMatchApiDetail({
    this.metadata,
    this.status,
    this.statusCode,
    this.result,
  });

  /// Convert FaceMatchApiDetail object to Map
  Map<String, dynamic> toMap() {
    return {
      'metadata': this.metadata,
      'status': this.status,
      'statusCode': this.statusCode,
      'result': this.result,
    };
  }

  /// Convert Map to FaceMatchApiDetail object
  factory FaceMatchApiDetail.fromMap(Map<dynamic, dynamic> map) {
    debugPrint('inside FaceMatchApiDetail.fromMap');

    if (map.isEmpty) {
      return FaceMatchApiDetail();
    }
    return FaceMatchApiDetail(
      metadata: Metadata.fromMap(map['metadata'] ?? {}),
      status: map['status'] as String?,
      statusCode: (map['statusCode'] ?? '').toString(),
      result: FaceMatchResult.fromMap(map['result'] ?? {}),
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'FaceMatchApiDetail{metadata: $metadata, status: $status, statusCode: $statusCode, result: $result}';
  }
}

/// FaceMatchResult
class FaceMatchResult {
  /// Details of Face match result
  final FaceMatchApiStructure? details;

  /// Error
  final String? error;

  /// Summary
  final Summary? summary;

  FaceMatchResult({
    this.details,
    this.error,
    this.summary,
  });

  /// Convert FaceMatchResult object to Map
  Map<String, dynamic> toMap() {
    return {
      'details': this.details,
      'error': this.error,
      'summary': this.summary,
    };
  }

  /// Convert Map to FaceMatchResult object
  factory FaceMatchResult.fromMap(Map<dynamic, dynamic> map) {
    debugPrint('inside FaceMatchResult.fromMap');

    if (map.isEmpty) {
      return FaceMatchResult();
    }
    return FaceMatchResult(
      details: FaceMatchApiStructure.fromMap(map['details'] ?? {}),
      error: map['error'] as String?,
      summary: Summary.fromMap(map['summary'] ?? {}),
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'FaceMatchResult{details: $details, error: $error, summary: $summary}';
  }
}

/// FaceMatchApiStructure
class FaceMatchApiStructure {
  /// Match score
  final String? matchScore;

  /// Confidence
  final String? conf;

  /// Match
  final Fieldvalue? match;

  /// to be reviewed or not
  final String? toBeReviewed;

  /// Constructor
  FaceMatchApiStructure({
    this.matchScore,
    this.conf,
    this.match,
    this.toBeReviewed,
  });

  /// Convert FaceMatchApiStructure object to Map
  Map<String, dynamic> toMap() {
    return {
      'matchScore': this.matchScore,
      'conf': this.conf,
      'match': this.match,
      'toBeReviewed': this.toBeReviewed,
    };
  }

  /// Convert Map to FaceMatchApiStructure object
  factory FaceMatchApiStructure.fromMap(Map<dynamic, dynamic> map) {
    debugPrint('inside FaceMatchApiStructure.fromMap');

    if (map.isEmpty) {
      return FaceMatchApiStructure();
      return FaceMatchApiStructure();
    }
    return FaceMatchApiStructure(
      matchScore: (map['matchScore'] ?? '').toString(),
      conf: (map['conf'] ?? '').toString(),
      match: Fieldvalue.fromMap(map['match'] ?? {}),
      toBeReviewed: map['toBeReviewed'] as String?,
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'FaceMatchApiStructure{matchScore: $matchScore, conf: $conf, match: $match, toBeReviewed: $toBeReviewed}';
  }
}
